/*
* save.c
* save Lua values as bytecode
* Luiz Henrique de Figueiredo <lhf@tecgraf.puc-rio.br>
* This code is hereby placed in the public domain.
*/

#define luac_c
#define LUA_CORE

#include "lua.h"
#include "lauxlib.h"

#include "ldo.h"
#include "ldebug.h"
#include "lfunc.h"
#include "lmem.h"
#include "lobject.h"
#include "lopcodes.h"
#include "lstring.h"
#include "lundump.h"

static const char save[]=
#include "save.h"
;

static int emit(lua_State* L)
{
 static Proto* f;
 static int pc=0;
 int n,i,j,k;
 Closure *c;
 switch (*lua_tostring(L,1))
 {
  case 'B':
	f=luaF_newproto(L);
	f->source=luaS_newliteral(L,"=(saved)");
	pc=0;
	n=lua_tointeger(L,2);
	f->k=luaM_newvector(L,n,TValue);
	f->sizek=n;
	n=lua_tointeger(L,3);
	f->code=luaM_newvector(L,n,Instruction);
	f->sizecode=n;
	n=lua_tointeger(L,4);
printf("B nf=%d\n",n);
	f->p=luaM_newvector(L,n,Proto*);
	f->sizep=n;
	break;
  case 'E':
	n=lua_tointeger(L,2);
	f->code[pc++]=CREATE_ABC(OP_RETURN,0,n+1,0);
	f->maxstacksize=lua_tointeger(L,3);
	f->sizecode=lua_tointeger(L,4);
#if 1
luaU_print(f,0);
if (!luaG_checkcode(f)) luaL_error(L,"bad code");
#endif
	c=luaF_newLclosure(L,0,hvalue(gt(L)));
	c->l.p=f;
	setclvalue(L,L->top-1,c);
	return 1;
  case 'N':
	n=lua_tointeger(L,2);
	f->code[pc++]=CREATE_ABC(OP_NEWTABLE,n,0,0);
	break;
  case 'L':
	n=lua_tointeger(L,2);
	k=lua_tointeger(L,3);	if (k<0) k=-k-1;
	f->code[pc++]=CREATE_ABx(OP_LOADK,n,k);
	break;
  case 'C':
	n=lua_tointeger(L,2);
	k=lua_tointeger(L,3);
	f->code[pc++]=CREATE_ABx(OP_CLOSURE,n,k);
	f->p[n]=clvalue(&L->base[2])->l.p;
	break;
  case 'S':
	i=lua_tointeger(L,2);
	j=lua_tointeger(L,3);	if (j<0) j=RKASK(-j-1);
	k=lua_tointeger(L,4);	if (k<0) k=RKASK(-k-1);
	f->code[pc++]=CREATE_ABC(OP_SETTABLE,i,j,k);
	break;
  case 'K':
	i=lua_tointeger(L,2);	if (i>=0) break;
	i=lua_tointeger(L,2);	if (i<0) i=-i-1;
	f->k[i]=L->base[2];
	break;
 }
 return 0;
}

LUALIB_API int luaopen_save (lua_State *L)
{
 if (luaL_loadbuffer(L,save,sizeof(save)-1,"@save.lua")) lua_error(L);
 lua_pushcfunction(L,emit);
 lua_call(L,1,1);
 lua_setglobal(L,"save");
 return 0;
}
